# Code for "smart" prediction in Version 1.7.1 (2003-06-16) of R

# Functions/expressions in this file are:

# The lm, predict.lm, predict.mlm, and glm functions here are
# based on R version 1.7.1
# Changes and new functions supplied here are
# (C) 2000, 2001, 2002, 2003 T. W. Yee and T. J. Hastie
# See DISCLAIMER for disclaimer and copyright notice.

# Version 1.1-1
# 9 July 2003

# Functions/expressions in this file are:

# lm
# predict.lm
# predict.mlm
# glm
# New functions =========================================================
# smart.mode.is
# setup.smart
# wrapup.smart
# get.smart.prediction
# put.smart
# get.smart
# is.smart
# # Smart functions below here ==========================================
# bs
# ns
# poly
# scale.default
# # Home baked smart functions below here ===============================
# my1   (example)
# my2   (example)
# stdze1 (example)
# stdze2 (example)


lm = 
function (formula, data = list(), subset, weights, na.action, 
    method = "qr", model = TRUE, x = FALSE, y = FALSE, qr = TRUE, 
    singular.ok = TRUE, contrasts = NULL, offset = NULL, smart=TRUE, ...) 
{
    ret.x <- x
    ret.y <- y

    if(smart) setup.smart("write")

    cl <- match.call()
    mf <- match.call(expand.dots = FALSE)
    mf$singular.ok <- mf$model <- mf$method <- NULL
    mf$x <- mf$y <- mf$qr <- mf$contrasts <- mf$smart <- mf$... <- NULL
    mf$drop.unused.levels <- TRUE
    mf[[1]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    if (method == "model.frame") 
        return(mf)
    else if (method != "qr") 
        warning("method = ", method, " is not supported. Using \"qr\".")
    mt <- attr(mf, "terms")
    na.act <- attr(mf, "na.action")
    xvars <- as.character(attr(mt, "variables"))[-1]
    if ((yvar <- attr(mt, "response")) > 0) 
        xvars <- xvars[-yvar]
    xlev <- if (length(xvars) > 0) {
        xlev <- lapply(mf[xvars], levels)
        xlev[!sapply(xlev, is.null)]
    }
    if (!singular.ok) 
        warning("only `singular.ok = TRUE' is currently implemented.")
    y <- model.response(mf, "numeric")
    w <- model.weights(mf)
    offset <- model.offset(mf)
    if (!is.null(offset) && length(offset) != NROW(y)) 
        stop("Number of offsets is ", length(offset), ", should equal ", 
            NROW(y), " (number of observations)")
    if (is.empty.model(mt)) {
        x <- NULL
        z <- list(coefficients = numeric(0), residuals = y, fitted.values = 0 * 
            y + offset, weights = w, rank = 0, df.residual = length(y))
        class(z) <- if (is.matrix(y)) 
            c("mlm.null", "lm.null", "mlm", "lm")
        else c("lm.null", "lm")
    }
    else {
        x <- model.matrix(mt, mf, contrasts)
        z <- if (is.null(w)) 
            lm.fit(x, y, offset = offset, ...)
        else lm.wfit(x, y, w, offset = offset, ...)
        class(z) <- c(if (is.matrix(y)) "mlm", "lm")
    }
    if (!is.null(na.act)) 
        z$na.action <- na.act
    z$offset <- offset
    z$contrasts <- attr(x, "contrasts")
    z$xlevels <- xlev
    z$call <- cl
    z$terms <- mt
    if (model) 
        z$model <- mf
    if (ret.x) 
        z$x <- x
    if (ret.y) 
        z$y <- y

    if(smart) {
        z$smart.prediction <- get.smart.prediction()
        wrapup.smart()
    }

    z
}
attr(lm, "smart") <- TRUE



glm = 
function (formula, family = gaussian, data = list(), weights = NULL, 
    subset = NULL, na.action = na.fail, start = NULL, etastart = NULL, 
    mustart = NULL, offset = NULL, control = glm.control(...), 
    model = TRUE, method = "glm.fit", x = FALSE, y = TRUE, contrasts = NULL, 
    smart = TRUE, ...) 
{
    call <- match.call()
    if (is.character(family)) 
        family <- get(family)
    if (is.function(family)) 
        family <- family()
    if (is.null(family$family)) {
        print(family)
        stop("`family' not recognized")
    }
    if (missing(data)) 
        data <- environment(formula)

    if(smart) setup.smart("write")

    mf <- match.call(expand.dots = FALSE)
    mf$family <- mf$start <- mf$control <- mf$maxit <- NULL
    mf$model <- mf$method <- mf$x <- mf$y <- mf$contrasts <- NULL
    mf$smart <- mf$... <- NULL
    mf$drop.unused.levels <- TRUE
    mf[[1]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())
    switch(method, model.frame = return(mf), glm.fit = 1, glm.fit.null = 1, 
        stop("invalid `method': ", method))
    mt <- attr(mf, "terms")
    na.act <- attr(mf, "na.action")
    xvars <- as.character(attr(mt, "variables"))[-1]
    if ((yvar <- attr(mt, "response")) > 0) 
        xvars <- xvars[-yvar]
    xlev <- if (length(xvars) > 0) {
        xlev <- lapply(mf[xvars], levels)
        xlev[!sapply(xlev, is.null)]
    }
    X <- if (!is.empty.model(mt)) 
        model.matrix(mt, mf, contrasts)
    Y <- model.response(mf, "numeric")
    weights <- model.weights(mf)
    offset <- model.offset(mf)
    if (!is.null(weights) && any(weights < 0)) 
        stop("Negative wts not allowed")
    if (!is.null(offset) && length(offset) != NROW(Y)) 
        stop("Number of offsets is ", length(offset), ", should equal ", 
            NROW(Y), " (number of observations)")
    fit <- (if (is.empty.model(mt)) 
        glm.fit.null
    else glm.fit)(x = X, y = Y, weights = weights, start = start, 
        etastart = etastart, mustart = mustart, offset = offset, 
        family = family, control = control, intercept = attr(mt, 
            "intercept") > 0)
    if (any(offset) && attr(mt, "intercept") > 0) {
        fit$null.deviance <- if (is.empty.model(mt)) 
            fit$deviance
        else glm.fit(x = X[, "(Intercept)", drop = FALSE], y = Y, 
            weights = weights, offset = offset, family = family, 
            control = control, intercept = TRUE)$deviance
    }
    if (model) 
        fit$model <- mf
    if (!is.null(na.act)) 
        fit$na.action <- na.act
    if (x) 
        fit$x <- X
    if (!y) 
        fit$y <- NULL
    fit <- c(fit, list(call = call, formula = formula, terms = mt, 
        data = data, offset = offset, control = control, method = method, 
        contrasts = attr(X, "contrasts"), xlevels = xlev))
    class(fit) <- c(if (is.empty.model(mt)) "glm.null", "glm", "lm")

    if(smart) {
        fit$smart.prediction <- get.smart.prediction()
        wrapup.smart()
    }

    fit
}
attr(glm, "smart") <- TRUE



predict.lm = 
function (object, newdata, se.fit = FALSE, scale = NULL, df = Inf, 
    interval = c("none", "confidence", "prediction"), level = 0.95, 
    type = c("response", "terms"), terms = NULL, ...) 
{
    # Smart prediction: handle the prediction flaw
    setup.smart("read", smart.prediction=object$smart.prediction)

    tt <- terms(object)
    if (missing(newdata) || is.null(newdata)) {
        mm <- X <- model.matrix(object)
        mmDone <- TRUE
        offset <- object$offset
    } else {
        X = model.matrix(delete.response(tt),newdata,contrasts = object$contrasts,
            xlev = object$xlevels)
        offset <- if (!is.null(off.num <- attr(tt, "offset"))) 
            eval(attr(tt, "variables")[[off.num + 1]], newdata)
        else if (!is.null(object$offset)) 
            eval(object$call$offset, newdata)
        mmDone <- FALSE
    }

    if(length(object$smart.prediction)) {
        wrapup.smart()
    }

    n <- NROW(object$qr$qr)
    p <- object$rank
    p1 <- 1:p
    piv <- object$qr$pivot[p1]
    if (p < ncol(X) && !(missing(newdata) || is.null(newdata))) 
        warning("prediction from a rank-deficient fit may be misleading")
    beta <- object$coefficients
    predictor <- drop(X[, piv, drop = FALSE] %*% beta[piv])
    if (!is.null(offset)) 
        predictor <- predictor + offset
    interval <- match.arg(interval)
    type <- match.arg(type)
    if (se.fit || interval != "none") {
        res.var <- if (is.null(scale)) {
            r <- object$resid
            f <- object$fitted
            w <- object$weights
            rss <- sum(if (is.null(w)) 
                r^2
            else r^2 * w)
            df <- n - p
            rss/df
        }
        else scale^2
        if (type != "terms") {
            XRinv <- if (missing(newdata) && is.null(w)) 
                qr.Q(object$qr)[, p1, drop = FALSE]
            else X[, piv] %*% qr.solve(qr.R(object$qr)[p1, p1])
            ip <- drop(XRinv^2 %*% rep(res.var, p))
        }
    }
    if (type == "terms") {
        if (!mmDone) {
            mm <- model.matrix(object)
            mmDone <- TRUE
        }
        aa <- attr(mm, "assign")
        ll <- attr(tt, "term.labels")
        if (attr(tt, "intercept") > 0) 
            ll <- c("(Intercept)", ll)
        aaa <- factor(aa, labels = ll)
        asgn <- split(order(aa), aaa)
        hasintercept <- attr(tt, "intercept") > 0
        if (hasintercept) {
            asgn$"(Intercept)" <- NULL
            if (!mmDone) {
                mm <- model.matrix(object)
                mmDone <- TRUE
            }
            avx <- colMeans(mm)
            termsconst <- sum(avx[piv] * beta[piv])
        }
        nterms <- length(asgn)
        predictor <- matrix(ncol = nterms, nrow = NROW(X))
        dimnames(predictor) <- list(rownames(X), names(asgn))
        if (se.fit || interval != "none") {
            ip <- matrix(ncol = nterms, nrow = NROW(X))
            dimnames(ip) <- list(rownames(X), names(asgn))
            Rinv <- qr.solve(qr.R(object$qr)[p1, p1])
        }
        if (hasintercept) 
            X <- sweep(X, 2, avx)
        unpiv <- rep(0, NCOL(X))
        unpiv[piv] <- p1
        for (i in seq(1, nterms, length = nterms)) {
            iipiv <- asgn[[i]]
            ii <- unpiv[iipiv]
            iipiv[ii == 0] <- 0
            predictor[, i] <- if (any(iipiv) > 0) 
                X[, iipiv, drop = FALSE] %*% beta[iipiv]
            else 0
            if (se.fit || interval != "none") 
                ip[, i] <- if (any(iipiv) > 0) 
                  as.matrix(X[, iipiv, drop = FALSE] %*% Rinv[ii, 
                    , drop = FALSE])^2 %*% rep(res.var, p)
                else 0
        }
        if (!is.null(terms)) {
            predictor <- predictor[, terms, drop = FALSE]
            if (se.fit) 
                ip <- ip[, terms, drop = FALSE]
        }
        attr(predictor, "constant") <- if (hasintercept) 
            termsconst
        else 0
    }
    if (interval != "none") {
        tfrac <- qt((1 - level)/2, df)
        hwid <- tfrac * switch(interval, confidence = sqrt(ip), 
            prediction = sqrt(ip + res.var))
        if (type != "terms") {
            predictor <- cbind(predictor, predictor + hwid %o% 
                c(1, -1))
            colnames(predictor) <- c("fit", "lwr", "upr")
        }
        else {
            lwr <- predictor + hwid
            upr <- predictor - hwid
        }
    }
    if (se.fit) 
        se <- sqrt(ip)
    if (missing(newdata) && !is.null(na.act <- object$na.action)) {
        predictor <- napredict(na.act, predictor)
        if (se.fit) 
            se <- napredict(na.act, se)
    }
    if (type == "terms" && interval != "none") {
        if (missing(newdata) && !is.null(na.act)) {
            lwr <- napredict(na.act, lwr)
            upr <- napredict(na.act, upr)
        }
        list(fit = predictor, se.fit = se, lwr = lwr, upr = upr, 
            df = df, residual.scale = sqrt(res.var))
    }
    else if (se.fit) 
        list(fit = predictor, se.fit = se, df = df, residual.scale = sqrt(res.var))
    else predictor
}



predict.glm = 
function (object, newdata = NULL, type = c("link", "response", 
    "terms"), se.fit = FALSE, dispersion = NULL, terms = NULL, ...) 
{
    if(length(object$smart.prediction)) {
        # Smart prediction: handle the prediction flaw
        setup.smart("read", smart.prediction=object$smart.prediction)
    }

    type <- match.arg(type)
    na.act <- object$na.action
    object$na.action <- NULL
    if (!se.fit) {
        if (missing(newdata)) {
            pred <- switch(type, link = object$linear.predictors, 
                response = object$fitted, terms = predict.lm(object, 
                  se.fit = se.fit, scale = 1, type = "terms", 
                  terms = terms))
            if (!is.null(na.act)) 
                pred <- napredict(na.act, pred)
        }
        else {
            pred <- predict.lm(object, newdata, se.fit, scale = 1, 
                type = ifelse(type == "link", "response", type), 
                terms = terms)
            switch(type, response = {
                pred <- family(object)$linkinv(pred)
            }, link = , terms = )
        }
    }
    else {
        if (inherits(object, "survreg")) 
            dispersion <- 1
        if (is.null(dispersion) || dispersion == 0) 
            dispersion <- summary(object, dispersion = dispersion)$dispersion
        residual.scale <- as.vector(sqrt(dispersion))
        pred <- predict.lm(object, newdata, se.fit, scale = residual.scale, 
            type = ifelse(type == "link", "response", type), 
            terms = terms)
        fit <- pred$fit
        se.fit <- pred$se.fit
        switch(type, response = {
            se.fit <- se.fit * abs(family(object)$mu.eta(fit))
            fit <- family(object)$linkinv(fit)
        }, link = , terms = )
        if (missing(newdata) && !is.null(na.act)) {
            fit <- napredict(na.act, fit)
            se.fit <- napredict(na.act, se.fit)
        }
        pred <- list(fit = fit, se.fit = se.fit, residual.scale = residual.scale)
    }
    pred
}



predict.mlm = 
function (object, newdata, se.fit = FALSE, ...) 
{
    if(length(object$smart.prediction)) {
        # Smart prediction: handle the prediction flaw
        setup.smart("read", smart.prediction=object$smart.prediction)
    }

    if (missing(newdata)) 
        return(object$fitted)
    if (se.fit) 
        stop("The 'se.fit' argument is not yet implemented for mlm objects")
    if (missing(newdata)) {
        X <- model.matrix(object)
        offset <- object$offset
    }
    else {
        tt <- terms(object)
        X = model.matrix(delete.response(tt),newdata,contrasts = object$contrasts,
            xlev = object$xlevels)
        offset <- if (!is.null(off.num <- attr(tt, "offset"))) 
            eval(attr(tt, "variables")[[off.num + 1]], newdata)
        else if (!is.null(object$offset)) 
            eval(object$call$offset, newdata)
    }
    piv <- object$qr$pivot[1:object$rank]
    pred <- X[, piv, drop = FALSE] %*% object$coefficients[piv, 
        ]
    if (!is.null(offset)) 
        pred <- pred + offset
    if (inherits(object, "mlm")) 
        pred
    else pred[, 1]
}
attr(predict.mlm, "smart") <- TRUE






# New functions =========================================================


smart.mode.is <- function(mode.arg) {
    if(missing(mode.arg)) { 
        if(exists(".smart.prediction", inher=T)) {
            .smart.prediction.mode
        } else {
            "neutral"
        }
    } else {
        if(exists(".smart.prediction", inher=T)) {
            .smart.prediction.mode == mode.arg
        } else {
            mode.arg=="neutral"
        }
    }
}


setup.smart <- function(mode.arg, smart.prediction=NULL, max.smart=30) {
    actual <- if(mode.arg=="write") vector("list", max.smart) else 
              if(mode.arg=="read") smart.prediction else 
              stop("value of mode.arg unrecognized")

    wrapup.smart()  # make sure

    if(length(actual)) {
        # Double check that smart.prediction is not trivial (in "read" mode)
        # If it is trivial then ignore it. This saves testing whether 
        # length(object$smart.prediction) > 0 in the predict methods function
        .smart.prediction <<- actual
        .smart.prediction.counter <<- 0
        .smart.prediction.mode <<- mode.arg
        .max.smart <<- max.smart
    }
}

wrapup.smart <- function() {
    while(exists(".smart.prediction", inher=T))
        rm(".smart.prediction", inherits=TRUE)
    while(exists(".smart.prediction.counter", inher=T))
        rm(".smart.prediction.counter", inherits=TRUE)
    while(exists(".smart.prediction.mode", inher=T))
        rm(".smart.prediction.mode", inherits=TRUE)
    while(exists(".max.smart", inher=T))
        rm(".max.smart", inherits=TRUE)
}


get.smart.prediction <- function() {

    smart.prediction.counter <- .smart.prediction.counter
    max.smart <- .max.smart

    if(smart.prediction.counter > 0) {
        # Save this on the object for smart prediction later
        smart.prediction <- .smart.prediction
        if(max.smart >= (smart.prediction.counter+1))
            for(i in max.smart:(smart.prediction.counter+1))
                smart.prediction[[i]] <- NULL
        smart.prediction
    } else 
        NULL
}


put.smart <- function(smart) {

    # Puts the info, if possible, in frame 1.
    # Doesn't returns whether it did it or not. 

#   if(ret <- smart.mode.is("write")) {

        # Write the info to frame 0 as well
        max.smart <- .max.smart
        smart.prediction.counter <- .smart.prediction.counter
        smart.prediction <- .smart.prediction
        smart.prediction.counter <- smart.prediction.counter + 1

        if(smart.prediction.counter > max.smart) {
            # if list is too small, make it larger
            max.smart <- max.smart + (inc.smart <- 10) # can change inc.smart
            smart.prediction <- c(smart.prediction, vector("list", inc.smart))
            .max.smart <<- max.smart
        }

        smart.prediction[[smart.prediction.counter]] <- smart
        .smart.prediction <<- smart.prediction
        .smart.prediction.counter <<- smart.prediction.counter
#   }
#   ret 
}


get.smart <- function() {
    # Returns one list component of information
    smart.prediction <- .smart.prediction
    smart.prediction.counter <- .smart.prediction.counter
    smart.prediction.counter <- smart.prediction.counter + 1
    .smart.prediction.counter <<- smart.prediction.counter
    smart <- smart.prediction[[smart.prediction.counter]]
    smart
}

smart.expression <- expression({

        # This expression only works if the first argument of the smart
        # function is "x", e.g., smartfun(x, ...)
        # Nb. .smart.match.call is the name of the smart function.

        smart  <- get.smart()
        .smart.prediction.mode <<- "neutral"

        .smart.match.call = as.character(smart$match.call)
        smart$match.call = NULL  # Kill it off for the do.call 

        ans.smart <- do.call(.smart.match.call, c(list(x=x), smart))
        .smart.prediction.mode <<- "read"

        ans.smart
})

is.smart <- function(fun) {
    if(is.logical(a <- attr(fun, "smart"))) a else FALSE 
}


# # Smart functions below here ==========================================

# Call "library(splines)" here rather than having 
# Depends: R (>= 1.7.0), splines
# in the "DESCRIPTION" file because of the search() ordering. 
library(splines) 

bs = 
function (x, df = NULL, knots = NULL, degree = 3, intercept = FALSE, 
    Boundary.knots = NULL) 
{
    # Note. the Boundary.knots are actually range(x), but are NULL here
    # because of smart prediction.

    x <- x  # Evaluate x

    if(smart.mode.is("read")) {
        return(eval(smart.expression))
    }


    nx <- names(x)
    x <- as.vector(x)
    nax <- is.na(x)
    if (nas <- any(nax)) 
        x <- x[!nax]
    if (is.null(Boundary.knots)) {
        outside <- FALSE
        Boundary.knots <- range(x)
    } else {
        Boundary.knots <- sort(Boundary.knots)
        outside <- (ol <- x < Boundary.knots[1]) | (or <- x > Boundary.knots[2])
    }
    ord <- 1 + (degree <- as.integer(degree))
    if (ord <= 1) 
        stop("degree must be integer >= 1")
    if (!is.null(df) && is.null(knots)) {
        nIknots <- df - ord + (1 - intercept)
        if (nIknots < 0) {
            nIknots <- 0
            warning("df was too small; have used  ", ord - (1 - intercept))
        }
        knots <- if (nIknots > 0) {
            knots <- seq(from = 0, to = 1, length = nIknots + 
                2)[-c(1, nIknots + 2)]
            quantile(x[!outside], knots)
        }
    }
    Aknots <- sort(c(rep(Boundary.knots, ord), knots))
    if (any(outside)) {
        warning("Some x values beyond boundary knots may cause ill-conditioned bases")
        derivs <- 0:degree
        scalef <- gamma(1:ord)
        basis <- array(0, c(length(x), length(Aknots) - degree - 1))
        if (any(ol)) {
            k.pivot <- Boundary.knots[1]
            xl <- cbind(1, outer(x[ol] - k.pivot, 1:degree, "^"))
            tt <- spline.des(Aknots, rep(k.pivot, ord), ord, derivs)$design
            basis[ol, ] <- xl %*% (tt/scalef)
        }
        if (any(or)) {
            k.pivot <- Boundary.knots[2]
            xr <- cbind(1, outer(x[or] - k.pivot, 1:degree, "^"))
            tt <- spline.des(Aknots, rep(k.pivot, ord), ord, 
                derivs)$design
            basis[or, ] <- xr %*% (tt/scalef)
        }
        if (any(inside <- !outside)) 
            basis[inside, ] <- spline.des(Aknots, x[inside], 
                ord)$design
    } else basis <- spline.des(Aknots, x, ord)$design
    if (!intercept) 
        basis <- basis[, -1, drop = FALSE]
    n.col <- ncol(basis)
    if (nas) {
        nmat <- matrix(NA, length(nax), n.col)
        nmat[!nax, ] <- basis
        basis <- nmat
    }
    dimnames(basis) <- list(nx, 1:n.col)
    a <- list(degree = degree, knots = if (is.null(knots)) numeric(0) else knots, 
        Boundary.knots = Boundary.knots, intercept = intercept)
    attributes(basis) <- c(attributes(basis), a)
    class(basis) <- c("bs", "basis")

    if(smart.mode.is("write"))
        put.smart(list(degree=degree,
                       intercept=intercept,
                       df=df, knots=knots,
                       Boundary.knots=Boundary.knots,
                       match.call=match.call()))

    basis
}
attr(bs, "smart") <- TRUE


ns = 
function (x, df = NULL, knots = NULL, intercept = FALSE,
          Boundary.knots = NULL) 
{
    # Note. the Boundary.knots are actually range(x), but are NULL here
    # because of smart prediction.

    x <- x  # Evaluate x

    if(smart.mode.is("read")) {
        return(eval(smart.expression))
    }


    nx <- names(x)
    x <- as.vector(x)
    nax <- is.na(x)
    if (nas <- any(nax)) 
        x <- x[!nax]
    if (is.null(Boundary.knots)) {
        outside <- FALSE
        Boundary.knots <- range(x)
    } else {
        Boundary.knots <- sort(Boundary.knots)
        outside <- (ol <- x < Boundary.knots[1]) | (or <- x > Boundary.knots[2])
    }
    if (!is.null(df) && is.null(knots)) {
        nIknots <- df - 1 - intercept
        if (nIknots < 0) {
            nIknots <- 0
            warning(paste("df was too small; have used ", 1 + 
                intercept))
        }
        knots <- if (nIknots > 0) {
            knots <- seq(0, 1, length = nIknots + 2)[-c(1, nIknots + 
                2)]
            quantile(x[!outside], knots)
        }
    }
    else nIknots <- length(knots)
    Aknots <- sort(c(rep(Boundary.knots, 4), knots))
    if (any(outside)) {
        basis <- array(0, c(length(x), nIknots + 4))
        if (any(ol)) {
            k.pivot <- Boundary.knots[1]
            xl <- cbind(1, x[ol] - k.pivot)
            tt <- spline.des(Aknots, rep(k.pivot, 2), 4, c(0, 
                1))$design
            basis[ol, ] <- xl %*% tt
        }
        if (any(or)) {
            k.pivot <- Boundary.knots[2]
            xr <- cbind(1, x[or] - k.pivot)
            tt <- spline.des(Aknots, rep(k.pivot, 2), 4, c(0, 
                1))$design
            basis[or, ] <- xr %*% tt
        }
        if (any(inside <- !outside)) 
            basis[inside, ] <- spline.des(Aknots, x[inside], 
                4)$design
    } else basis <- spline.des(Aknots, x, 4)$design
    const <- spline.des(Aknots, Boundary.knots, 4, c(2, 2))$design
    if (!intercept) {
        const <- const[, -1, drop = FALSE]
        basis <- basis[, -1, drop = FALSE]
    }
    qr.const <- qr(t(const))
    basis <- as.matrix((t(qr.qty(qr.const, t(basis))))[, -(1:2)])
    n.col <- ncol(basis)
    if (nas) {
        nmat <- matrix(NA, length(nax), n.col)
        nmat[!nax, ] <- basis
        basis <- nmat
    }
    dimnames(basis) <- list(nx, 1:n.col)
    a <- list(degree = 3, knots = if (is.null(knots)) numeric(0) else knots, 
        Boundary.knots = Boundary.knots, intercept = intercept)
    attributes(basis) <- c(attributes(basis), a)
    class(basis) <- c("ns", "basis")

    if(smart.mode.is("write"))
        put.smart(list(df=ncol(basis),
                       knots=knots,
                       intercept=intercept,
                       Boundary.knots=Boundary.knots,
                       match.call=match.call()))

    basis
}
attr(ns, "smart") <- TRUE


poly = 
function (x, ..., degree = 1, coefs = NULL) 
{
    x <- x  # Evaluate x

    if(smart.mode.is("read")) {
        smart <- get.smart()
        degree <- smart$degree
        coefs  <- smart$coefs
    }


    dots <- list(...)
    if (nd <- length(dots)) {
        if (nd == 1 && length(dots[[1]]) == 1) 
            degree <- dots[[1]]
        else return(polym(x, ..., degree = degree))
    }
    if (is.matrix(x)) {
        m <- unclass(as.data.frame(cbind(x, ...)))
        return(do.call("polym", c(m, degree = degree)))
    }
    if (degree < 1) 
        stop("degree must be at least 1")

    # At prediction time x may be less than the degree
    if(smart.mode.is("write") || smart.mode.is("neutral"))
    if (degree >= length(x))
        stop("degree must be less than number of points")


    n <- degree + 1
    if (is.null(coefs)) {
        if (degree >= length(x)) 
            stop("degree must be less than number of points")
        xbar <- mean(x)
        x <- x - xbar
        X <- outer(x, seq(length = n) - 1, "^")
        QR <- qr(X)
        z <- QR$qr
        z <- z * (row(z) == col(z))
        raw <- qr.qy(QR, z)
        norm2 <- colSums(raw^2)
        alpha <- (colSums(x * raw^2)/norm2 + xbar)[1:degree]
        Z <- raw/rep(sqrt(norm2), each = length(x))
        colnames(Z) <- 1:n - 1
        Z <- Z[, -1, drop = FALSE]
        attr(Z, "degree") <- 1:degree
        attr(Z, "coefs") <- list(alpha = alpha, norm2 = c(1, 
            norm2))
        class(Z) <- c("poly", "matrix")
    } else {
        alpha <- coefs$alpha
        norm2 <- coefs$norm2
        Z <- matrix(, length(x), n)
        Z[, 1] <- 1
        Z[, 2] <- x - alpha[1]
        if (degree > 1) 
            for (i in 2:degree) Z[, i + 1] <- (x - alpha[i]) * 
                Z[, i] - (norm2[i + 1]/norm2[i]) * Z[, i - 1]
        Z <- Z/rep(sqrt(norm2[-1]), each = length(x))
        colnames(Z) <- 0:degree
        Z <- Z[, -1, drop = FALSE]
    }

    if(smart.mode.is("write"))
        put.smart(list(degree=degree, coefs=attr(Z, "coefs"),
                       match.call=match.call()))

    return(Z)
}
attr(poly, "smart") <- TRUE


scale.default = 
function (x, center = TRUE, scale = TRUE) 
{
    x <- as.matrix(x)

    if(smart.mode.is("read")) {
        return(eval(smart.expression))
    }


    nc <- ncol(x)
    if (is.logical(center)) {
        if (center) {
            center <- colMeans(x, na.rm = TRUE)
            x <- sweep(x, 2, center)
        }
    } else if (is.numeric(center) && (length(center) == nc)) 
        x <- sweep(x, 2, center)
    else stop("Length of center must equal the number of columns of x")
    if (is.logical(scale)) {
        if (scale) {
            f <- function(v) {
                v <- v[!is.na(v)]
                sqrt(sum(v^2)/max(1, length(v) - 1))
            }
            scale <- apply(x, 2, f)
            x <- sweep(x, 2, scale, "/")
        }
    } else if (is.numeric(scale) && length(scale) == nc) 
        x <- sweep(x, 2, scale, "/")
    else stop("Length of scale must equal the number of columns of x")
    if (is.numeric(center)) 
        attr(x, "scaled:center") <- center
    if (is.numeric(scale)) 
        attr(x, "scaled:scale") <- scale

    if(smart.mode.is("write")) {
        put.smart(list(center=center, scale=scale,
                       match.call=match.call()))
    }

    x
}
attr(scale.default, "smart") <- TRUE






# # Home baked smart functions below here ===============================


# "my1" is an example 

"my1" <- function(x, minx=min(x)) {

    x <- x   # Evaluate x

    if(smart.mode.is("read")) {
        smart  <- get.smart()
        minx <- smart$minx          # Overwrite its value 
    } else 
    if(smart.mode.is("write"))
        put.smart(list(minx=minx))

    (x-minx)^2
}
attr(my1, "smart") <- TRUE



# "my2" is equivalent to "my1". This format is useful if there are
# many arguments

"my2" <- function(x, minx=min(x)) {

    x <- x   # Evaluate x

    if(smart.mode.is("read")) {
        return(eval(smart.expression))
    } else 
    if(smart.mode.is("write"))
        put.smart(list(minx=minx, match.call=match.call()))

    (x-minx)^2
}

attr(my2, "smart") <- TRUE


# stdze1 is another (useful) example. It standardizes a variable. 


"stdze1" <- function(x, center=TRUE, scale=TRUE) {

    x <- x  # Evaluate x

    if(!is.vector(x))
        stop("x must be a vector")

    if(smart.mode.is("read")) {
        smart  <- get.smart()
        return((x-smart$center)/smart$scale)
    }

    if(is.logical(center))
        center <- if(center) mean(x) else 0
    if(is.logical(scale))
        scale <- if(scale) sqrt(var(x)) else 1

    if(smart.mode.is("write"))
        put.smart(list(center=center,
                       scale=scale))
    # Normal use
    (x-center)/scale
}
attr(stdze1, "smart") <- TRUE

"stdze2" <- function(x, center=TRUE, scale=TRUE) {

    x <- x  # Evaluate x

    if(!is.vector(x))
        stop("x must be a vector")

    if(smart.mode.is("read")) {
        return(eval(smart.expression))
    }

    if(is.logical(center))
        center <- if(center) mean(x) else 0
    if(is.logical(scale))
        scale <- if(scale) sqrt(var(x)) else 1

    if(smart.mode.is("write"))
        put.smart(list(center=center,
                       scale=scale,
                       match.call=match.call()))

    (x-center)/scale
}
attr(stdze2, "smart") <- TRUE



