\name{get.smart}
\alias{get.smart}
\title{
Retrieve one component of ".smart.prediction"
}
\description{
Retrieve one component of the list \code{.smart.prediction} from
the workspace (R) or 
frame 1 (S-PLUS).
}
\usage{
get.smart()
}
\value{
Returns with one list component of \code{.smart.prediction} from
the workspace (R)
or
frame 1 (S-PLUS),
in fact, \code{.smart.prediction[[.smart.prediction.counter]]}.
The whole procedure mimics a first-in first-out stack (better known
as a queue).
}
\section{Side Effects}{
The variable \code{.smart.prediction.counter} in
the workspace (R)
or
frame 1 (S-PLUS)
is incremented beforehand, and then written back to
the workspace (R)
or
frame 1 (S-PLUS).
}
\details{
\code{get.smart} is used in "read" mode within a smart function:
it retrieves parameters saved at the time of fitting, and
is used for prediction.
\code{get.smart} is only used in smart functions such as 
\code{poly};
\code{get.smart.prediction} is only used in modelling functions
such as \code{lm} and \code{glm}.
\code{get.smart} gets only a part of \code{.smart.prediction} whereas
\code{get.smart.prediction} gets the entire \code{.smart.prediction}.
}
\seealso{
get.smart.prediction
}
\examples{
# Here is a smart function

"my1" <- function(x, minx=min(x)) {

    x <- x

    if(smart.mode.is("read")) {
        smart  <- get.smart()
        minx <- smart$minx          # Overwrite its value
    } else
    if(smart.mode.is("write"))
        put.smart(list(minx=minx))

    sqrt(x-minx)
}
attr(my1, "smart") <- TRUE 
}
%\keyword{smart}
\keyword{models}
\keyword{regression}
\keyword{programming}

% Converted by Sd2Rd version 1.10.
